var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
export class MemoizedPromise {
    /**
     * Create a new object that memoizes promises returned by a user-defined function.
     *
     * @param userDefinedFunction user-defined function that returns a promise.
     * @param cacheIndefinitely if true, indefinitely cache the promise.
     */
    constructor(userDefinedFunction, cacheIndefinitely) {
        this.cache = {};
        this.hit = 0;
        this.miss = 0;
        this.error = 0;
        this.cacheIndefinitely = cacheIndefinitely === true;
        this.userDefinedFunction = userDefinedFunction;
        this.request = this.request.bind(this);
    }
    request(_a) {
        var { cacheKey } = _a, params = __rest(_a, ["cacheKey"]);
        if (!cacheKey) {
            this.error++;
            return Promise.reject(new Error('Unique key not passed'));
        }
        if (this.cache[cacheKey]) {
            this.hit++;
            return this.cache[cacheKey];
        }
        this.cache[cacheKey] = this.userDefinedFunction(params)
            .then((res) => {
            if (!this.cacheIndefinitely) {
                delete this.cache[cacheKey];
            }
            return res;
        })
            .catch((err) => {
            delete this.cache[cacheKey];
            throw err;
        });
        this.miss++;
        return this.cache[cacheKey];
    }
    logStats() {
        console.log('# errors : ' + this.error);
        console.log('# entries : ' + (this.hit + this.miss));
        console.log('# hits : ' + this.hit);
        console.log('# misses : ' + this.miss);
        if (this.hit > 0) {
            console.log('hit rate : ' + (this.hit / (this.hit + this.miss)));
        }
        if (this.miss > 0) {
            console.log('miss rate : ' + (this.miss / (this.hit + this.miss)));
        }
    }
}
const cache = (function () {
    let map_ = {};
    let hit_ = 0;
    let miss_ = 0;
    const put = (namespace, key, value) => {
        if (!hasKey(namespace, key)) {
            miss_++;
            map_[namespace] = map_[namespace] || {};
            map_[namespace][key] = { data: value, hit: 0, touchedAt: new Date() };
        }
    };
    const get = (namespace, key) => {
        if (!hasKey(namespace, key)) {
            return null;
        }
        hit_++;
        map_[namespace][key].hit++;
        map_[namespace][key].touchedAt = new Date();
        return map_[namespace][key].data;
    };
    const hasKey = (namespace, key) => {
        return namespace in map_ && key in map_[namespace];
    };
    const size = () => {
        return hit_ + miss_;
    };
    const invalidate = () => {
        logStats();
        map_ = {};
        hit_ = 0;
        miss_ = 0;
    };
    const logStats = () => {
        console.log('# entries : ' + (hit_ + miss_));
        console.log('# hits : ' + hit_);
        console.log('# misses : ' + miss_);
        if (hit_ > 0) {
            console.log('hit rate : ' + (hit_ / (hit_ + miss_)));
        }
        if (miss_ > 0) {
            console.log('miss rate : ' + (miss_ / (hit_ + miss_)));
        }
    };
    const temporarilyMemoizedPromise = (fn) => {
        return new MemoizedPromise(fn, false);
    };
    const indefinitelyMemoizedPromise = (fn) => {
        return new MemoizedPromise(fn, true);
    };
    return {
        put: put,
        get: get,
        hasKey: hasKey,
        size: size,
        invalidate: invalidate,
        logStats: logStats,
        temporarilyMemoizedPromise: temporarilyMemoizedPromise,
        indefinitelyMemoizedPromise: indefinitelyMemoizedPromise,
    };
})();
export { cache };
