import { Autocomplete, ListOfItems } from './autocomplete';
/**
 * Wrapper around an autocomplete over concepts.
 *
 * Example:
 * ```html
 * <!DOCTYPE html>
 * <html lang="en">
 * <head>
 *   <meta charset="UTF-8">
 * </head>
 * <body>
 *   <div style="width: 50%">
 *     <autocomplete-concept
 *       popup-width="600px"
 *       popup-height="auto"
 *       popup-max-items="15"
 *       placeholder="Enter an address..."
 *       concept="address"
 *       filter-properties="NUMERO_DE_RUE,RUE,COMPLEMENT_DE_RUE,CODE_POSTAL,VILLE"
 *       title-properties="NUMERO_DE_RUE,RUE,COMPLEMENT_DE_RUE,CODE_POSTAL,VILLE"
 *       subtitle-properties="NUMERO_DE_RUE,RUE,COMPLEMENT_DE_RUE,CODE_POSTAL,VILLE"></autocomplete-concept>
 *   </div>
 *   <div>
 *     The popup will be displayed above this text.
 *   </div>
 * </body>
 * <script src="https://unpkg.com/@computablefacts/sdk-javascript/dist/bundle/index.js"></script>
 * <script>
 *   cf.webComponents.registerAutocompleteConcept()
 * </script>
 * </html>
 * ```
 */
declare class AutocompleteConcept extends Autocomplete {
    protected UNKNOWN_CONCEPT: string;
    protected UNKNOWN_PROPERTIES: string;
    protected UNKNOWN_MAX_ITEMS: string;
    constructor();
    newListOfItems(uuid: number, text: string, caret: number): Promise<ListOfItems>;
}
export { AutocompleteConcept };
