/**
 * The list of items to display in the popup.
 */
declare type ListOfItems = {
    uuid: number;
    elements: HTMLElement[];
};
/**
 * Wrapper around an autocomplete.
 *
 * This component is not meant to be instantiated but to be subclassed.
 */
declare class Autocomplete extends HTMLElement {
    protected SELECT_VALUE: string;
    protected THERE_ARE_NO_RESULTS_FOR_YOUR_SEARCH: string;
    protected POPUP_WIDTH: string;
    protected POPUP_HEIGHT: string;
    protected uuid: number;
    private timeout;
    protected static extractTermBeforeCaret(text: string, caret: number): string;
    protected static extractTermsBeforeCaret(text: string, caret: number): string[];
    private static moveSelection;
    private static selectedItem;
    private static previousItem;
    private static nextItem;
    private static firstItem;
    private static lastItem;
    private static position;
    private static insertSelectedItem;
    constructor();
    get value(): string;
    set value(newValue: string);
    connectedCallback(): void;
    /**
     * Create a single `HTMLElement` of type `<li>` to replace the ones displayed in the popup. This `HTMLElement`
     * must represent a single list item.
     *
     * @param value the list item `value`.
     * @param html the list item visible name.
     * @param fact an optional fact with additional information.
     * @returns the newly created `HTMLElement`.
     */
    newListItem(value: string, html: string, fact?: string): HTMLElement;
    /**
     * Create a list of `HTMLElement` to replace the ones displayed in the popup. This list of `HTMLElement`
     * must represent the full list.
     *
     * @param uuid a unique identifier in order to be able to apply updates in the right order.
     * @param text the text in the `<input>` field of the `Autocomplete` component.
     * @param caret the caret position in the text.
     * @returns a `Promise<ListOfItems>`.
     */
    newListOfItems(uuid: number, text: string, caret: number): Promise<ListOfItems>;
    /**
     * Execute `getAttribute` and returns a default value if the attribute does not exist or is `null`
     * or equals to the empty string.
     *
     * @param attribute the attribute name.
     * @param defaultValue the default value to return.
     * @protected
     */
    protected getAttributeOrDefault(attribute: string, defaultValue: string): string;
    /**
     * Ensure calls to `newListOfItems` are not performed on each keystroke.
     *
     * @param func the function to execute.
     * @param delay the delay in milliseconds to wait for before executing the function.
     * @private
     */
    private debounce;
}
export { Autocomplete, ListOfItems };
